==> mcss-0.0.31.tar.gz: mcss-0.0.31/mcss/MulticompartmentGillespie.cpp <==
// multicompartment gillespie direct method class definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "MulticompartmentGillespie.h"

#include <limits>
#include <ecsb/ecsb.h>

using namespace Simulation;
using namespace ECSB;

// constructor
MulticompartmentGillespie::MulticompartmentGillespie(const unsigned long seed) \
		: SimulationAlgorithm("Multicompartment Gillespie", seed) { }

// initialise algorithm
int MulticompartmentGillespie::preprocess(void)
{
	register unsigned long i;
	register double tautmp;
	Compartment *compartment;

	// calculate tau and mu for each compartment
	// and find compartment with smallest tau
	tau0 = TAUMAX;
	for(i = 0; i < num_compartments; i++) {
		compartment = psystem->getCompartment(i);
		psystem->execute(i);
		tautmp = compartment->getTimeToNextReaction();
		// keep information about compartment with smallest tau
		if(tautmp != 0.0 && tautmp < tau0) {
			tau0 = tautmp;
			i0 = i;
			mu0 = compartment->getNextReaction();
			rs0 = compartment->getNextRuleSet();
		}
	}

	if(tau0 == TAUMAX)
		return 0;  // no reactions to execute
		
	return 1;
}

// perform one iteration of algorithm for all compartments
// using multicompartment gillespie algorithm
int MulticompartmentGillespie::execute(void)
{
	register unsigned long i, it, i0new = 0, mu0new = 0, rs0new = 0;
	register double tau0new, tautmp;
	register Compartment *compartment;

	// update simulation time
	advanceTime(tau0);

	// perform reaction in selected compartment
	compartment = psystem->getCompartment(i0);
	psystem->executeRule(rs0, mu0, i0);
	
	// calculate next waiting time
	tau0new = TAUMAX;
	
	// run gillespie to recalculate next reaction in selected compartment
	psystem->execute(i0);
	// make next reaction if time to next reaction the smallest
	tautmp = compartment->getTimeToNextReaction();
	if(tautmp != 0.0 && tautmp < tau0new) {
		tau0new = tautmp;
		i0new = i0;
		mu0new = compartment->getNextReaction();
		rs0new = compartment->getNextRuleSet();
	}

	// update target compartment if channel
	it = num_compartments;
	if(compartment->isChannel(rs0, mu0)) {
		it = compartment->getRuleTarget(rs0, mu0);
		// run gillespie to recalculate next reaction in target compartment
		psystem->execute(it);
		// make next reaction if time to next reaction the smallest
		compartment = psystem->getCompartment(it);
		tautmp = compartment->getTimeToNextReaction();
		if(tautmp != 0.0 && tautmp < tau0new) {
			tau0new = tautmp;
			i0new = it;
			mu0new = compartment->getNextReaction();
			rs0new = compartment->getNextRuleSet();
		}
	}

	// calculate next reaction
	for(i = 0; i < num_compartments; i++) {
		if(!(i == i0 || i == it)) {
			compartment = psystem->getCompartment(i);
			if((tautmp = compartment->getTimeToNextReaction()) == 0.0)
				continue;
			compartment->setTimeToNextReaction(tautmp -= tau0);
			// keep compartment with smallest time to next reaction
			if(tautmp != 0.0 && tautmp < tau0new) {
				tau0new = tautmp;
				i0new = i;
				mu0new = compartment->getNextReaction();
				rs0new = compartment->getNextRuleSet();
			}
		}
	}

	// finish if no more reactions to execute
	if(tau0new == TAUMAX) {
		tau0 = mu0 = i0 = rs0 = 0;
		return 0;
	}

	// set next reaction
	tau0 = tau0new;
	i0 = i0new;
	mu0 = mu0new;
	rs0 = rs0new;

	return 1;
}

// perform one iteration of algorithm for one compartment
// using gillespie direct method
void MulticompartmentGillespie::execute(unsigned long c)
{
	register unsigned long i, mu0, rs0;
	register double a0, tau0, r1, r2, am;
	register Compartment *compartment;

	// don't run for compartments with no rules
//	if(!compartment->getNumRules())
//		return;

	compartment = psystem->getCompartment(c);
	
	// calculate propensities
	a0 = 0.0;
	for(rs0 = 0; rs0 < compartment->getNumRuleSets(); rs0++)
		for(i = 0; i < compartment->getNumRules(rs0); i++)
			a0 += calculatePropensity(rs0, i, c);

	// finish if zero level
	if(a0 == 0.0) {
		compartment->setTimeToNextReaction(0.0);
		compartment->setNextReaction(0);
		compartment->setNextRuleSet(0);
		return;
	}

	// generate time to next reaction
	r1 = randomNumber();
	tau0 = -1.0 * log(r1) / a0;

	// select next reaction
	r2 = randomNumber();
	am = r2 * a0;
	for(rs0 = 0; rs0 < compartment->getNumRuleSets(); rs0++)
		for(mu0 = 0; mu0 < compartment->getNumRules(rs0); mu0++) {
			am -= compartment->getPropensity(rs0, mu0);
			if(am <= 0.0)
				goto done;
		}
		
	done:
		compartment->setTimeToNextReaction(tau0);
		compartment->setNextReaction(mu0);
		compartment->setNextRuleSet(rs0);
}

==> mcss-0.0.39.tar.gz: mcss-0.0.39/mcss/MulticompartmentGillespie.cpp <==
// multicompartment gillespie direct method class definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "MulticompartmentGillespie.h"

#include <limits>
#include <ecsb/ecsb.h>
#include <gsl/gsl_sf_log.h>

using namespace Simulation;
using namespace ECSB;

// constructor
MulticompartmentGillespie::MulticompartmentGillespie(const unsigned long seed) \
		: SimulationAlgorithm("Multicompartment Gillespie", seed) { }

// initialise algorithm
int MulticompartmentGillespie::preprocess(void)
{
	register unsigned long i;
	register double tautmp;
	Compartment *compartment;

	// calculate tau and mu for each compartment
	// and find compartment with smallest tau
	tau0 = TAUMAX;
	for(i = 0; i < num_compartments; i++) {
		compartment = psystem->getCompartment(i);
		psystem->execute(i);
		tautmp = compartment->getTimeToNextReaction();
		// keep information about compartment with smallest tau
		if(tautmp != 0.0 && tautmp < tau0) {
			tau0 = tautmp;
			i0 = i;
			mu0 = compartment->getNextReaction();
			rs0 = compartment->getNextRuleSet();
		}
	}

	if(tau0 == TAUMAX)
		return 0;  // no reactions to execute

	return 1;
}

// perform one iteration of algorithm for all compartments
// using multicompartment gillespie algorithm
int MulticompartmentGillespie::execute(void)
{
	register unsigned long i, it, i0new = 0, mu0new = 0, rs0new = 0;
	register double tau0new, tautmp;
	register Compartment *compartment;

	// update simulation time
	advanceTime(tau0);

	// perform reaction in selected compartment
	compartment = psystem->getCompartment(i0);
	psystem->executeRule(rs0, mu0, i0);

	// calculate next waiting time
	tau0new = TAUMAX;

	// run gillespie to recalculate next reaction in selected compartment
	psystem->execute(i0);
	// make next reaction if time to next reaction the smallest
	tautmp = compartment->getTimeToNextReaction();
	if(tautmp != 0.0 && tautmp < tau0new) {
		tau0new = tautmp;
		i0new = i0;
		mu0new = compartment->getNextReaction();
		rs0new = compartment->getNextRuleSet();
	}

	// update target compartment if channel
	it = num_compartments;
	if(compartment->isChannel(rs0, mu0)) {
		it = compartment->getRuleTarget(rs0, mu0);
		// run gillespie to recalculate next reaction in target compartment
		psystem->execute(it);
		// make next reaction if time to next reaction the smallest
		compartment = psystem->getCompartment(it);
		tautmp = compartment->getTimeToNextReaction();
		if(tautmp != 0.0 && tautmp < tau0new) {
			tau0new = tautmp;
			i0new = it;
			mu0new = compartment->getNextReaction();
			rs0new = compartment->getNextRuleSet();
		}
	}

	// calculate next reaction
	for(i = 0; i < num_compartments; i++) {
		if(!(i == i0 || i == it)) {
			compartment = psystem->getCompartment(i);
			if((tautmp = compartment->getTimeToNextReaction()) == 0.0)
				continue;
			compartment->setTimeToNextReaction(tautmp -= tau0);
			// keep compartment with smallest time to next reaction
			if(tautmp != 0.0 && tautmp < tau0new) {
				tau0new = tautmp;
				i0new = i;
				mu0new = compartment->getNextReaction();
				rs0new = compartment->getNextRuleSet();
			}
		}
	}

	// finish if no more reactions to execute
	if(tau0new == TAUMAX) {
		tau0 = mu0 = i0 = rs0 = 0;
		return 0;
	}

	// set next reaction
	tau0 = tau0new;
	i0 = i0new;
	mu0 = mu0new;
	rs0 = rs0new;

	return 1;
}

// perform one iteration of algorithm for one compartment
// using gillespie direct method
void MulticompartmentGillespie::execute(unsigned long c)
{
	register unsigned long i, mu0, rs0;
	register double a0, tau0, r1, r2, am;
	register Compartment *compartment;

	// don't run for compartments with no rules
//	if(!compartment->getNumRules())
//		return;

	compartment = psystem->getCompartment(c);

	// calculate propensities
	a0 = 0.0;
	for(rs0 = 0; rs0 < compartment->getNumRuleSets(); rs0++)
		for(i = 0; i < compartment->getNumRules(rs0); i++)
			a0 += calculatePropensity(rs0, i, c);

	// finish if zero level
	if(a0 == 0.0) {
		compartment->setTimeToNextReaction(0.0);
		compartment->setNextReaction(0);
		compartment->setNextRuleSet(0);
		return;
	}

	// generate time to next reaction
	r1 = randomNumberPos();
	tau0 = -1.0 * gsl_sf_log(r1) / a0;

	// select next reaction
	r2 = randomNumberPos();
	am = r2 * a0;
	for(rs0 = 0; rs0 < compartment->getNumRuleSets(); rs0++)
		for(mu0 = 0; mu0 < compartment->getNumRules(rs0); mu0++) {
			am -= compartment->getPropensity(rs0, mu0);
			if(am <= 0.0)
				goto done;
		}

	done:
		compartment->setTimeToNextReaction(tau0);
		compartment->setNextReaction(mu0);
		compartment->setNextRuleSet(rs0);
}
